/**
 * Introduction
 *
 * @title ROVLib - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file canvas.h
 *
 * @type core
 *
 * @description
 *
 * The canvas core system maintains windows, performing all common functions
 * while allowing overrides for many of them.
 *
 * The core system will translate all WIMP (screen) coordinates into offsets
 * from the canvas origin, so if a mouse click event occurs, the mouse click
 * handler is called with the coordinates changed so that they are relative
 * to the canvas origin. This also filters through to the drawing subsystem
 * so that during a canvas update or redraw, all drawing is relative to the
 * canvas origin. This saves a lot of repetitive calculation.
 *
 * The normal steps in creating and displaying a canvas are as follows:
 * @list create the canvas
 * @list set the canvas colours
 * @list set the canvas flags
 * @list set the canvas handlers
 * @list make the canvas into a window
 * @list display the canvas
 *
 * In addition, WIMP window handles are translated into canvas handles where
 * it is appropriate.
 * 
 * Note that the icon core system requires the canvas core system in order to
 * function.
 */

#ifndef __rov_canvas
#define __rov_canvas


#ifdef __cplusplus
extern "C"
{
#endif

#ifndef __rov_main
#include "main.h"
#endif

/**
 * Definition of a canvas open event (similar to the Wimp_OpenWindow event).
 * @field x0 the left hand (screen) coordinate of the canvas.
 * @field y0 the bottom (screen) coordinate of the canvas.
 * @field x1 the right hand (screen) coordinate of the canvas.
 * @field y1 the top (screen) coordinate of the canvas (excluding the title
 * bar).
 * @field scx the scroll bar x position.
 * @field scy the scroll bar y position.
 * @field behind the WIMP window handle of the window that this canvas is
 * behind.
 * @field flags the canvas flags.
 * @see Canvas flags
 */
typedef struct {
  canvas c;
  int x0;
  int y0;
  int x1;
  int y1;
  int scx;
  int scy;
  int behind;
  int flags;
} canvas_openstr;

/**
 * Definition of a canvas redraw event (similar to the Wimp_GetRectangle
 * response).
 * @field c the canvas being redrawn
 * @field x0 the left hand (screen) coordinate of the canvas.
 * @field y0 the bottom (screen) coordinate of the canvas.
 * @field x1 the right hand (screen) coordinate of the canvas.
 * @field y1 the top (screen) coordinate of the canvas (excluding the title
 * bar).
 * @field scx the scroll bar x position.
 * @field scy the scroll bar y position.
 * @field gx0 the left hand (canvas) coordinate of the area being redrawn.
 * @field gy0 the bottom (canvas) coordinate of the area being redrawn.
 * @field gx1 the right hand (canvas) coordinate of the area being redrawn.
 * @field gy1 the left (canvas) coordinate of the area being redrawn.
 */
typedef struct {
  canvas c;
  int x0;
  int y0;
  int x1;
  int y1;
  int scx;
  int scy;
  int gx0;
  int gy0;
  int gx1;
  int gy1;
} canvas_redrawstr;

/**
 * Definition of a canvas mouse event (similar to the Wimp_MouseClick event).
 * @field x the (canvas) x coordinate of the mouse event.
 * @field y the (canvas) y coordinate of the mouse event.
 * @field b the mouse buttons being pressed.
 * @field w the WIMP window handle.
 * @field i the WIMP icon handle.
 */
typedef struct {
  int x;
  int y;
  int b;
  int w;
  int i;
} canvas_mousestr;

/**
 * Definition of a canvas caret event (similar to the Wimp_Caret event).
 * @field w the WIMP window handle.
 * @field i the WIMP icon handle.
 * @field x the (canvas) x coordinate.
 * @field y the (canvas) y coordinate.
 * @field flags the caret flags.
 * @field index the index into the character string.
 */
typedef struct {
  int w;
  int i;
  int x;
  int y;
  int flags;
  int index;
} canvas_caretstr;

/**
 * Definition of a canvas key event (similar to the Wimp_Key event).
 * @field w the WIMP window handle.
 * @field i the WIMP icon handle.
 * @field x the (canvas) x coordinate.
 * @field y the (canvas) y coordinate.
 * @field flags the caret flags.
 * @field index the index into the character string.
 * @field chcode the character code.
 */
typedef struct {
  int w;
  int i;
  int x;
  int y;
  int flags;
  int index;
  int chcode;
} canvas_keystr;

/**
 * Initialises the canvas core system. This must be done before any canvasses
 * are initialised.
 */
void canvas_init(void);

/**
 * Create a canvas giving width and height and title
 * @param width the width (in OS units).
 * @param height the height (in OS units).
 * @param title the title of the canvas.
 * @return the created canvas.
 * @note By default, the origin is bottom left. However, if the width is
 * negative, the origin changes from the left to the right. Likewise, if
 * the height is negative, the origin changes from the bottom to the top.
 * @note This only creates the canvas structure in memory, but does not
 * turn it into a WIMP window. This allows colours and certain attributes
 * to be changed that must be changed before the window has been created.
 */
canvas canvas_create(int width,
		     int height,
		     char *title);

/**
 * Create a canvas using a pre-created, but uninitialised canvas.
 * @param width is the width
 * @param height is the height
 * @param title is the title.
 * @param c is the canvas.
 * @return the created canvas.
 * @note This isn't designed for use by user programs.
 * @see canvas_create()
 * @see canvas_removeallocated()
 */
canvas canvas_createallocated(int width,
			      int height,
			      char *title,
			      canvas c);

/**
 * Finds a canvas based on the WIMP window handle.
 * @param handle is the window handle
 * @return the canvas to which it refers, or NULL if not found.
 */
canvas canvas_find(int handle);

#ifndef uint
/**
 * Definition of uint
 */
#define uint unsigned int
#endif

/**
 * Sets the colour of a canvas
 * @param c the canvas to set the colour of.
 * @param col the colour (WIMP colour number, 255 = 'transparent').
 * @note The canvas must not have been made into a window.
 * @see canvas_setoutlinecolour()
 * @see canvas_setcaretcolour()
 */
void canvas_colour(canvas c,
		   int col);

/**
 * Sets the outline colour of a canvas
 * @param c the canvas to set the outline colour of.
 * @param col the colour (WIMP colour number, 255 = 'none').
 * @note The canvas must not have been made into a window.
 * @see canvas_colour()
 * @see canvas_setcaretcolour()
 */
void canvas_setoutlinecolour(canvas c,
			     int col);

/**
 * Sets the canvas highlighted colour.
 * @param c the canvas to set the highlighted colour
 * @param col the colour to set it to (WIMP colour number).
 * @note The canvas must not have been made into a window.
 * @see canvas_colour()
 * @see canvas_setoutlinecolour()
 */
void canvas_setcaretcolour(canvas c,
			   int col);

/**
 * Sets the window flags of the canvas
 * @param c the canvas to set the flags
 * @param bic the inverse AND value of the flags
 * @param eor the EOR value of the flags
 * @note The flags is changed according to the following algorihtm:
 * new = (old AND NOT (bic)) EOR eor.
 * @note The canvas click style flags are held in bits 16 to 19.
 * @note The canvas must not have been made into a window.
 * @see Canvas flags
 * @see Canvas click styles
 */
void canvas_setflags(canvas c,
		     uint bic,
		     uint eor);

/**
 * Sets the sprite area pointer of the canvas.
 * @param c the canvas to set the sprite area of.
 * @param spr_area the pointer to the sprite area
 * @note The canvas must not have been made into a window.
 */
void canvas_setspritearea(canvas c,
			  void *spr_area);

/**
 * Makes the canvas into a window.
 * @param c the canvas to make into a window
 * @see canvas_makeatlocation()
 * @see canvas_makeatlocationsize()
 */
void canvas_make(canvas c);

/**
 * Makes the canvas into a window, giving a default start location.
 * @param c the canvas to make into a window.
 * @param x the left hand x coordinate of the window (in screen coordinates).
 * @param y the bottom x coordinate of the window (in screen coordinates).
 * @note This is primarily used to allow the resize-to-full icon to position
 * the window at certain coordinates if the window has always been full size.
 * @see canvas_make()
 * @see canvas_makeatlocationsize()
 * @see canvas_reopen()
 */
void canvas_makeatlocation(canvas c,
			   int x,
			   int y);

/**
 * Makes the canvas into a window, giving a default start location and size.
 * @param c the canvas to make into a window.
 * @param x the left hand x coordinate of the window (in screen coordinates).
 * @param y the bottom x coordinate of the window (in screen coordinates).
 * @param w the width of the window.
 * @param h the height of the window.
 * @see canvas_make()
 * @see canvas_makeatlocation()
 * @see canvas_reopen()
 */
void canvas_makeatlocationsize(canvas c,
			       int x,
			       int y,
			       int w,
			       int h);

/**
 * Sets the pointer to the canvas title to your own area, rather than the
 * canvas system's own allocated area.
 * @param c the canvas to set the title pointer.
 * @param title the text pointer of the canvas title.
 * @note This allows you to extend the memory used to over the 64 bytes that
 * is reserved for each canvas. However, it means that the canvas_settitle()
 * function will not change your memory's text, but you can still use the
 * function to redraw the title.
 * @see canvas_settitle()
 */
void canvas_settitlelocation(canvas c,
			     char *title);

/**
 * Sets the canvas' minimum size.
 * @param c the canvas.
 * @param width the minimum width.
 * @param height the maximum height.
 * @note The canvas must not have been made into a window.
 * @see canvas_setsize()
 * @see canvas_setfullsize()
 */
void canvas_setminsize(canvas c, 
		       int width, 
		       int height);

/**
 * Gets the WIMP window handle of the canvas.
 * @param c the canvas to get the handle.
 * @return the WIMP window handle.
 */
int canvas_handle(canvas c);

/**
 * Sets the maximum size of the canvas.
 * @param c the canvas to set the size.
 * @param width the width to set it to.
 * @param height the height to set it to.
 * @note If the canvas has not been created, then scrollbars are automatically
 * added to the canvas.
 * @see canvas_setminsize()
 * @see canvas_setfullsize()
 */
void canvas_setsize(canvas c,
		    int width,
		    int height);

/**
 * Sets the maximum size of the canvas, giving all bounds.
 * @param c the canvas to set the size.
 * @param x0 the left hand x coordinate.
 * @param y0 the bottom y coordinate.
 * @param x1 the right hand x coordinate.
 * @param y1 the top y coordinate
 * @note If the canvas has not been created, then scrollbars are automatically
 * added to the canvas.
 * @see canvas_setminsize()
 * @see canvas_setsize()
 */
void canvas_setfullsize(canvas c,
			int x0,
			int y0,
			int x1,
			int y1);

/**
 * Opens a canvas with the given open paramters.
 * @param c the canvas to open.
 * @param open_str the open parameters structure.
 * @see typedef struct canvas_openstr
 * @see canvas_opencentre()
 * @see canvas_opentop()
 * @see canvas_opencentrebottom()
 * @see canvas_reopen()
 */
void canvas_open(canvas c,
		 canvas_openstr *open_str);

/**
 * Closes a canvas.
 * @param c the canvas to close
 */
void canvas_close(canvas c);

/**
 * Closes a canvas, and removes it from the WIMP, and frees the memory
 * used.
 * @param c the canvas to remove.
 */
void canvas_remove(canvas c);

/**
 * Closes a canvas, removes it from the WIMP, but does not remove the
 * memory. It is the removal of canvas_createallocated().
 * @param c the canvas to create.
 * @see canvas_createallocated()
 */
void canvas_removeallocated(canvas c);

/**
 * Forces the redraw of the canvas'  display.
 * @param c the canvas to redraw
 */
void canvas_forceredraw(canvas c);

/**
 * Redraws a section of the canvas' display.
 * @param c the canvas to redraw.
 * @param x0 the left hand coordinate.
 * @param y0 the bottom coordinate.
 * @param x1 the right hand coordinate.
 * @param y1 the top coordinate.
 * @see canvas_forceredraw()
 * @see canvas_forceupdate()
 * @see canvas_update()
 * @see canvas_redrawhandler()
 */
void canvas_redraw(canvas c,
		   int x0,
		   int y0,
		   int x1,
		   int y1);

/**
 * Forces the update of the canvas' display.
 * @param c the canvas to redraw
 * @param ... [proc] the function to call to do the update.
 * @note The function is normally defined as:
 * void update_func(canvas c, int x0, int y0, int x1, int y1);
 * but can have a short cut of void update_func();
 * @note The canvas system calls Wimp_UpdateWindow and Wimp_GetRectangle
 * for you, and calls the function with each of the rectangles.
 * @see canvas_forceredraw()
 * @see canvas_redraw()
 * @see canvas_update()
 * @see canvas_redrawhandler()
 */
void canvas_forceupdate(canvas c,
			...);

/**
 * Updates the section  of the canvas' display.
 * @param c the canvas to redraw.
 * @param x0 the left hand coordinate.
 * @param y0 the bottom coordinate.
 * @param x1 the right hand coordinate.
 * @param y1 the top coordinate.
 * @param ... [proc] the function to call to do the update.
 * @note The function is normally defined as:
 * void update_func(canvas c, int x0, int y0, int x1, int y1);
 * but can have a short cut of void update_func();
 * @note The canvas system calls Wimp_UpdateWindow and Wimp_GetRectangle
 * for you, and calls the function with each of the rectangles.
 * @see canvas_forceredraw()
 * @see canvas_redraw()
 * @see canvas_forceupdate()
 * @see canvas_redrawhandler()
 */
void canvas_update(canvas c,
		   int x0,
		   int y0,
		   int x1,
		   int y1,
		   ...);

/**
 * Opens a canvas in the centre of the screen, at the top of the stack.
 * @param c the canvas to open.
 * @see canvas_open()
 * @see canvas_opentop()
 * @see canvas_opencentrebottom()
 * @see canvas_reopen()
 */
void canvas_opencentre(canvas c);

/**
 * Opens a canvas at the top of the stack, using the given coordinates.
 * @param c the canvas to open.
 * @param x the left hand screen coordinate.
 * @param y the bottom screen coordinate.
 * @see canvas_open()
 * @see canvas_opencentre()
 * @see canvas_opencentrebottom()
 * @see canvas_reopen()
 */
void canvas_opentop(canvas c,
		    int x,
		    int y);

/**
 * Opens a canvas in the centre of the screen, at the bottom of the stack.
 * @param c the canvas to open.
 * @see canvas_open()
 * @see canvas_opencentre()
 * @see canvas_opentop()
 * @see canvas_reopen()
 */
void canvas_opencentrebottom(canvas c);

/**
 * Opens a canvas at its current position, but at the top of the stack.
 * @param c the canvas to open.
 * @note If the canvas has not been opened before, then it is opened at its
 * 'default' position, which you can set using canvas_makeatlocation() and
 * canvas_makeatlocationsize().
 * @see canvas_makeatlocation()
 * @see canvas_makeatlocationsize()
 * @see canvas_open()
 * @see canvas_opencentre()
 * @see canvas_opentop()
 * @see canvas_opencentrebottom()
 */
void canvas_reopen(canvas c);

/**
 * Sets the title of the canvas, and redraws it (if the window is open).
 * @param c the canvas to redraw.
 * @param title the title to set it to.
 * @see canvas_settitlelocation()
 */
void canvas_settitle(canvas c,
		     char *title);

/**
 * Gets the state of the canvas.
 * @param c the canvas.
 * @param open_str the block to store the information.
 */
void canvas_getcanvasstate(canvas c,
			   canvas_openstr *open_str);

/**
 * Gets the coordinates of the mouse.
 * @param c the canvas.
 * @param xp the pointer to the integer to fill in with the x coordinate.
 * @param yp the pointer to the integer to fill in with the y coordinate.
 * @see canvas_getlocation()
 * @see canvas_getorigin()
 * @see canvas_getrawmouse()
 */
void canvas_getpointerlocation(canvas c,
			       int *xp,
			       int *yp);

/**
 * Converts screen coordinates into canvas coordinates.
 * @param c the canvas.
 * @param xp the pointer to the x coordinate to convert.
 * @param yp the pointer to the y coordinate to convert.
 * @see canvas_getpointerlocation()
 * @see canvas_getorigin()
 */
void canvas_getlocation(canvas c,
			int *xp,
			int *yp);

/**
 * Gets the origin of the canvas as screen coordinates.
 * @param c the canvas.
 * @param xp the pointer to the integer to fill in with the x coordinate.
 * @param yp the pointer to the integer to fill in with the y coordinate.
 * @see canvas_getpointerlocation()
 * @see canvas_getlocation()
 */
void canvas_getorigin(canvas c, 
		      int *xp,
		      int *yp);

/**
 * Gets the canvas to claim the caret.
 * @param c the canvas to claim the caret.
 */
void canvas_claimcaret(canvas c);

/**
 * Gets the raw mouse coordinates (i.e. screen coordinates, not canvas coordinates)
 * @param mouse_str the pointer to the block to fill in the coordinates
 * @see canvas_getpointerlocation()
 */
void canvas_getrawmouse(canvas_mousestr *mouse_str);

/**
 * Gets the unique ID handle of a canvas.
 * @param c the canvas to get the handle of.
 * @return the unique ID handle.
 * @note This can be a pointer cast as an integer to the information that is used
 * to display the canvas.
 * @see canvas_setidhandle()
 */
int canvas_idhandle(canvas c);

/**
 * Sets the unique ID handle of a canvas.
 * @param c the canvas to set the handle of.
 * @param id the unique ID handle
 * @note This can be a pointer cast as an integer to the information that is used
 * to display the canvas.
 * @see canvas_idhandle()
 */
void canvas_setidhandle(canvas c,
			int id);

/**
 * Opens a template file
 * @param file the name of the template file.
 * @return status - 0 for failure, 1 for success.
 * @note This is experimental, and has not been fully debugged.
 * @see canvas_templatecanvas()
 * @see canvas_templatesetspritearea()
 */
int canvas_templateopen(char *file);

/**
 * Creates a canvas from a template file.
 * @param name the name of the template in the template file
 * @note This is experimental, and has not been fully debugged.
 * @see canvas_templateopen()
 * @see canvas_templatesetspritearea()
 */
canvas canvas_templatecanvas(char *name);

/**
 * Sets the sprite area for subsequent template creation.
 * @param spr_area the pointer to the sprite area.
 * @note This is experimental, and has not been fully debugged.
 * @see canvas_templateopen()
 * @see canvas_templatecanvas()
 * @see canvas_setspritearea()
 */
void canvas_templatesetspritearea(void *spr_area);

/**
 * Gets the state of the caret
 * @param caret_str the pointer to the block to hold the caret state.
 */
void canvas_getcaretstate(canvas_caretstr *caret_str);

/**
 * Gets the raw window information of the canvas.
 * @param c the canvas to get the information of.
 * @return the pointer to the window informaton.
 * @note This is only used internally.
 */
void * canvas_windowptr(canvas c);

/**
 * This indicates what the current redraw mode is.
 * 0 means it is a redraw, 1 means it is an update.
 * @note This is normally used by redraw functions to determine how the redraw
 * should be performed.
 */
extern int canvas_redrawmode;

#ifdef OLDSTYLE
void * canvas_addhandler(canvas, int, void *);
#else
/**
 * This adds a handler to the particular canvas event.
 * @param c the canvas to add the handler to
 * @param event_id the event ID
 * @param ... the function
 * @return the old handler function pointer.
 * @see canvas_redrawhandler()
 * @see canvas_openhandler()
 * @see canvas_closehandler()
 * @see canvas_pointerhandler()
 * @see canvas_clickhandler()
 * @see canvas_keyhandler()
 * @see canvas_menuhandler()
 * @see canvas_carethandler()
 * @see canvas_loadhandler()
 * @see canvas_savehandler()
 * @see canvas_helphandler()
 * @see canvas_scrollhandler()
 * @see Canvas handler values
 */
void * canvas_addhandler(canvas c,
			 int event_id,
			 ...);
#endif

/**
 * This is the icon bar canvas. It is a static area of memory holding the handlers
 * for the icon bar.
 * @note Do not use. Use CANVAS_ICONBAR instead.
 * @see CANVAS_ICONBAR
 */
extern void * canvas_iconbar;

/**
 * This is the icon bar canvas. It is a static area of memory holding the handlers
 * for the icon bar, but with a different window handle (for creating icons).
 * @note Do not use. Use CANVAS_ICONBARRIGHT instead.
 * @see CANVAS_ICONBARRIGHT
 */
extern void * canvas_iconbarright;

/**
 * The canvas for the icon bar.
 * @see CANVAS_ICONBARRIGHT
 */
#define CANVAS_ICONBAR ((canvas)(&canvas_iconbar))


/**
 * The canvas for the right hand side of the icon bar (used to create icons)
 * @note This basically is the same as CANVAS_ICONBAR, except the window handle
 * is -2.
 * @see CANVAS_ICONBAR
 */
#define CANVAS_ICONBARRIGHT ((canvas)(&canvas_iconbarright))

/**
 * Definition of control codes 'A' thru 'Z'
 * @mparam j the ASCII uppercase character to get the control code for.
 * @note To use this, use if(k->chcode == ctrl('F'))...
 */
#define ctrl(j)	(j - 64)

/**
 * The null canvas
 */
#define canvas_null	(0)

/**
 * CANVAS_REDRAWHANDLER - called when the canvas needs redrawing.
 * @param c the canvas needing to be redrawn.
 * @param x0 the left hand (canvas) coordinate.
 * @param y0 the bottom (canvas) coordinate.
 * @param x1 the right hand (canvas) coordinate.
 * @param y1 the top (canvas) coordinate.
 * @note The canvas system calls Wimp_RedrawWindow and Wimp_GetRectangle for you, and
 * calls this function for each of the rectangles.
 * @note The default is not to perform any redrawing.
 * @note This must be called before the canvas is made into a window. The Wimp_Redraw
 * flags of the window is automatically set.
 * @see CANVAS_REDRAWHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_redrawhandler)(canvas c,
				      int x0,
				      int y0,
				      int x1,
				      int y1);

/**
 * CANVAS_OPENHANDLER - called when the canvas needs opening (moving,
 * resizing).
 * @param c the canvas that is being opened/moved.
 * @param open_str the block of information where it is being moved to.
 * @note The default is to open the canvas for you. Only override this default if your
 * canvas needs special movement rules.
 * @see CANVAS_OPENHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_openhandler)(canvas c,
				    canvas_openstr *open_str);
 
/**
 * CANVAS_CLOSEHANDLER - called when the canvas needs closing.
 * @param c the canvas being closed.
 * @note The default is to close the canvas for you. Only override this default if your
 * canvas needs special closing rules.
 * @see CANVAS_CLOSEHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_closehandler)(canvas c);

/**
 * CANVAS_POINTERHANDLER - called when the canvas enters or leaves the window.
 * @param c the canvas with the pointer entering or leaving.
 * @param enter gives whether the pointer is entering or leaving the canvas. Non-zero indicates
 * entering the canvas.
 * @note The default is to do nothing.
 * @see Pointer event values
 * @see CANVAS_POINTERHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_pointerhandler)(canvas c,
				       int enter);

/**
 * CANVAS_CLICKHANDLER - called when SELECT or ADJUST is clicked on the window.
 * @param c the canvas with the click event.
 * @param mouse_str the block of information holding the click event.
 * @note The default is to do nothing.
 * @see CANVAS_CLICKHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_clickhandler)(canvas c,
				     canvas_mousestr *mouse_str);

/**
 * CANVAS_KEYHANDLER - called when the canvas is issued a key event from the WIMP.
 * @param c the canvas with the key event.
 * @param key_str the block of information holding the key event.
 * @note The default is to do nothing.
 * @see CANVAS_KEYHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_keyhandler)(canvas c,
				   canvas_keystr *key_str);

/**
 * CANVAS_MENUHANDLER - called when MENU is clicked on the window
 * @param c the canvas with MENU being pressed in it.
 * @param mouse_str the block of information holding the click event.
 * @note The default is to do nothing.
 * @see CANVAS_MENUHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_menuhandler)(canvas c,
				    canvas_mousestr *mouse_str);

/**
 * CANVAS_CARETHANDLER - called when the caret is gained or lost.
 * @param c the canvas with the caret event.
 * @param caret_str the block of information holding the caret event.
 * @param gain gives whether the canvas is gaining or losing the caret. Non-zero indicates
 * gaining the caret.
 * @note The default is to do nothing.
 * @see Caret event values
 * @see CANVAS_CARETHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_carethandler)(canvas c,
				     canvas_caretstr *caret_str,
				     int gain);

/**
 * CANVAS_LOADHANDLER - called when the canvas is asked to load a file (typically
 * when a user drops a file onto it from the filer).
 * @param c the canvas being asked to load a file.
 * @param file_name the name of the file.
 * @param file_type the type of the file.
 * @param file_size the size of the file.
 * @note The default is to do nothing.
 * @see CANVAS_LOADHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_loadhandler)(canvas c,
				    char *file_name,
				    int file_type,
				    int file_size);

/**
 * CANVAS_SAVEHANDLER - called when the canvas is asked to save a file (typically
 * when a user drops a file onto it from another application).
 * @param c the canvas being asked to save a file.
 * @param file_name the name of the file.
 * @param file_type the type of the file.
 * @param file_size the size of the file.
 * @note The default is to do nothing.
 * @see CANVAS_SAVEHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_savehandler)(canvas c,
				    char *file_name,
				    int file_type,
				    int file_size);

/**
 * CANVAS_HELPHANDLER - called when !Help requires text to display.
 * @param c the canvas being requested for help.
 * @param mouse_str the block of information holding the help request event.
 * @note The default is to do nothing.
 * @see CANVAS_HELPHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_helphandler)(canvas c,
				    canvas_mousestr *mouse_str);

/**
 * CANVAS_SCROLLHANDLER - called when scrolling window, and the CANVAS_SCROLLREQUEST 
 * flag has been set.
 * @param c the canvas being requested for scrolling.
 * @param open_str the block of information holding the next open event.
 * @param xdir the scroll event x direction.
 * @param ydir the scroll event y direction.
 * @note The default is to do nothing.
 * @see Canvas scroll directions
 * @see CANVAS_SCROLLHANDLER
 * @see canvas_addhandler()
 */
typedef void (* canvas_scrollhandler)(canvas c,
				      canvas_openstr *open_str,
				      int xdir,
				      int ydir);

/**
 * @section Pointer event values
 * @see canvas_pointerhandler()
 */
#define CANVAS_POINTERLEFT	(0)
#define CANVAS_POINTERENTERED	(1)

/**
 * @section Caret event values
 * @see canvas_carethandler()
 */
#define CANVAS_LOSTCARET	(0)
#define CANVAS_GAINEDCARET	(1)

/**
 * @section Canvas flags
 * @see canvas_setflags()
 */
#define CANVAS_OLDTITLEBAR      (1<<0)
#define CANVAS_MOVEABLE		(1<<1)
#define CANVAS_OLDVSCROLL	(1<<2)
#define CANVAS_OLDHSCROLL	(1<<3)
#define CANVAS_AUTOREDRAW	(1<<4)
#define CANVAS_PANE		(1<<5)
#define CANVAS_DRAGOFFSCREEN	(1<<6)
#define CANVAS_OLDNOBACKCLOSE	(1<<7)
#define CANVAS_SCROLLREQUEST	(1<<8)
#define CANVAS_SCROLLNOREPEAT	(1<<9)
#define CANVAS_USEGCOL		(1<<10)
#define CANVAS_MAKEWINDOWBOTTOM	(1<<11)
#define CANVAS_GENERATEHOTKEYS	(1<<12)
#define CANVAS_FORCEONSCREEN	(1<<13)
#define CANVAS_IGNORERHEXTENT	(1<<14)
#define CANVAS_IGNOREBOTEXTENT	(1<<15)
#define CANVAS_OPENED		(1<<16)
#define CANVAS_FULLYVISIBLE	(1<<17)
#define CANVAS_OPENEDFULLSIZE	(1<<18)
#define CANVAS_TOGGLESIZED	(1<<19)
#define CANVAS_HASINPUTFOCUS	(1<<20)
#define CANVAS_FORCEDONSCREEN	(1<<21)
#define CANVAS_BACK		(1<<24)
#define CANVAS_CLOSE		(1<<25)
#define CANVAS_TITLEBAR		(1<<26)
#define CANVAS_TOGGLE		(1<<27)
#define CANVAS_VSCROLL		(1<<28)
#define CANVAS_RESIZE		(1<<29)
#define CANVAS_HSCROLL		(1<<30)
#define CANVAS_NEWFLAGS		(1<<31)
#define CANVAS_CONTROLSFLAGS	(0xff000000U)

/**
 * @section Canvas click styles
 * @see canvas_setflags()
 */
#define CANVAS_NOCLICKS		(0x00000)
#define CANVAS_CONTINUOUS	(0x10000)
#define CANVAS_AUTOREPEAT	(0x20000)
#define CANVAS_CLICK		(0x30000)
#define CANVAS_RELEASE		(0x40000)
#define CANVAS_DOUBLECLICK	(0x50000)
#define CANVAS_CLICKDRAG	(0x60000)
#define CANVAS_RELEASEDRAG	(0x70000)
#define CANVAS_DOUBECLICKDRAG	(0x80000)
#define CANVAS_CLICKDOUBLEDRAG	(0xa0000)
#define CANVAS_GAININPUT	(0xf0000)
#define CANVAS_MOUSEFLAGS	(0xf0000)

/**
 * @section Canvas handler values
 * @see canvas_addhandler()
 */
#define CANVAS_REDRAWHANDLER	(0)
#define CANVAS_OPENHANDLER	(1)
#define CANVAS_CLOSEHANDLER	(2)
#define CANVAS_POINTERHANDLER	(3)
#define CANVAS_CLICKHANDLER	(4)
#define CANVAS_KEYHANDLER	(5)
#define CANVAS_MENUHANDLER	(6)
#define CANVAS_CARETHANDLER	(7)
#define CANVAS_LOADHANDLER	(8)
#define CANVAS_SAVEHANDLER	(9)
#define CANVAS_HELPHANDLER	(10)
#define CANVAS_SCROLLHANDLER	(11)

/**
 * @section Canvas scroll directions
 * @see canvas_scrollhandler()
 */
#define CANVAS_SCROLLPAGELEFT	(-2)
#define CANVAS_SCROLLLEFT	(-1)
#define CANVAS_SCROLLNULL	(0)
#define CANVAS_SCROLLRIGHT	(1)
#define CANVAS_SCROLLPAGERIGHT	(2)
#define CANVAS_SCROLLPAGEDOWN	(-2)
#define CANVAS_SCROLLDOWN	(-1)
#define CANVAS_SCROLLNULL	(0)
#define CANVAS_SCROLLUP		(1)
#define CANVAS_SCROLLPAGEUP	(2)


#ifdef __cplusplus
}
#endif

#endif
